!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!! Program for the conversion of meteogram data to palm p3d file    !!!
!!! Last Modified: DBS (September 29, 2023)                          !!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	Character(len=100)	:: cbuffer
	Integer,Dimension(11)	:: izlevs
	Real,Dimension(50)	:: p, t, qv, qc, qi, rh, clc, clcon,   &
                                   u, v, ws, z, pt 

	Real,Dimension(11)	:: zg, ptzg, ptg, qvg, ug, vg, qvzg 

	Data izlevs /1, 3, 6, 8, 9, 13, 15, 18, 20, 21, 23/

	Open (1, file = 'M_TVM-14Jan2010', form='formatted')
	Open (2, file = 'INPUT.p3d',       form='formatted')
	Open (3, file = 'OUTPUT.p3d',      form='formatted')

!!! Gathering useful information from the meteogram file

	Read (1, 11) idd, imm, iyyyy, ihh, imn, isc, hsurf
 11	Format (/, 23x, 2(i2, 1x), i4, 2x, 3(i2, 1x), /, 18x, f10.3)

	Read (1, 12) alat, alon
 12	Format (2(/, 18x, f10.3))

	Read (1, 13) ps
 13	Format (4(/), 13x, f6.1, 4(/))

	Do i = 50, 1, -1

 101	Continue
	Read (1, *,err=101, end=100) k, p(i), t(i), qv(i), qc(i), qi(i), rh(i),&
                                     clc(i), clcon(i), u(i), v(i), ws(i), z(i)

	pt(i) = (t(i)+273.15)*((1000.0/p(i))**0.286)

	Enddo
 100	Continue

	Read (1, 14) z0
 14	Format (58(/), 38x, f13.5)

	Read (1, 15) qvs
 15	Format (2(/), 38x, f11.3)
	qvs = qvs/1000.0

	Read (1, 16) alai, plcov, rootdp
 16	Format (2(/), 3(38x, f11.3, /))

	Read (1, 17) ts, tg, tso0
 17	Format (1(/), 2(38x, f11.3, /), 3(/), 38x, f11.3)

	Read (1, 18) tso1, wso1
	Read (1, 18) tso2, wso2
	Read (1, 18) tso3, wso3
	Read (1, 18) tso4, wso4
	Read (1, 18) tso5, wso5
	Read (1, 18) tso6, wso6
	Read (1, 18) tso7, wso7
	Read (1, 18) tso8, wso8
 18	Format (38x, f11.3, 15x, f11.3)

	tso1	= tso1 + 273.15
	tso2	= tso2 + 273.15
	tso3	= tso3 + 273.15
	tso4	= tso4 + 273.15
	tso5	= tso5 + 273.15
	tso6	= tso6 + 273.15
	tso7	= tso7 + 273.15
	tso8	= tso8 + 273.15

!!! Processing of meteogram data to make it compatible to p3d requirements

	pt_surface = (ts+273.15)*((1000.0/ps)**0.286)

	Do i = 1, 11

	k = izlevs(i)

	zg(i) 	= z(k)
	ptzg(i) = pt(k)
	ug(i) 	= u(k)
	vg(i) 	= v(k)
	qvg(i) 	= qv(k)

	Enddo

	Do i = 1, 10

	ptg2 = (ptzg(i+1) - ptzg(i))
	zg2  = (zg(i+1) - zg(i))

	ptg(i) = (ptg2/zg2)*100.0

	qvg2 = (qvg(i+1) - qvg(i))
	qvzg(i) = (qvg2/zg2)*100.0

	Enddo

!!! Dumping the gathered data into p3d file

	Do i = 1, 20

	Read (2, 21) cbuffer
	Write (3, 21) cbuffer
 21	Format (a100)

	Enddo

	Read (2, 21) cbuffer
	Write (3, 22) pt_surface
 22	Format ('    pt_surface                 = ', f7.3)

	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 24) (ptg(i),i=1,10)
 24	Format ('    pt_vertical_gradient       = ', 5(f7.3, ','), /,       &
                33x, 5(f7.3, ','))

	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 25) hsurf, (zg(i),i=1,9)
 25	Format ('    pt_vertical_gradient_level = ', 5(f7.1, ','), /,       &
                33x, 5(f7.1, ','))

	Do i = 1, 3

	Read (2, 21) cbuffer
	Write (3, 21) cbuffer

	Enddo

	Read (2, 21) cbuffer
	Write (3, 26) '''', iyyyy, imm, idd, ihh, imn, isc, ''''
 26	Format ('    origin_date_time           = ', a1, i4.4, '-', i2.2, &
                '-', i2.2, 1x, i2.2, ':', i2.2, ':', i2.2, 1x, '+00', a1)

	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 27) alat, alon
 27	Format ('    latitude                   = ', f6.3, ',', /,         &
                '    longitude                  = ', f6.3, ',')
               
	Read (2, 21) cbuffer
	Write (3, 21) cbuffer
	
	Read (2, 21) cbuffer
	Write (3, 28) ps
 28	Format ('    surface_pressure           = ', f7.2, ',')

	Read (2, 21) cbuffer
	Write (3, 21) cbuffer

	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 29) 0.0, (ug(i), i=2,10)
 29	Format ('    u_profile                  = ', 5(f7.2, ',', 1x), /, &
                33x, 5(f7.2, ',', 1x))

	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 30) 0.0, (vg(i), i=2,10)
 30	Format ('    v_profile                  = ', 5(f7.2, ',', 1x), /, &
                33x, 5(f7.2, ',', 1x))

	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 31) 0.0, (zg(i), i=2,10)
 31	Format ('    uv_heights                 = ', 5(f7.2, ',', 1x), /, &
                33x, 5(f7.2, ',', 1x))

	Do i = 1, 5
	Read (2, 21) cbuffer
	Write (3, 21) cbuffer
	Enddo

	Read (2, 21) cbuffer
	Write (3, 32) z0
 32	Format ('    roughness_length           = ', f6.2, ',')

	Do i = 1, 7
	Read (2, 21) cbuffer
	Write (3, 21) cbuffer
	Enddo

	Read (2, 21) cbuffer
	Write (3, 33) qvs
 33	Format ('    q_surface                  = ', f9.6, ',')
	
	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 34) (qvzg(i), i=1,10)
 34	Format ('    q_vertical_gradient        = ', 5(f9.6, ',', 1x), /, &
                33x, 5(f9.6, ',', 1x))

	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Read (2, 21) cbuffer
	Write (3, 35) (zg(i), i=1,10)
 35	Format ('    q_vertical_gradient_level  = ', 5(f7.2, ',', 1x), /, &
                33x, 5(f7.2, ',', 1x))

	Do i = 1, 75
	Read (2, 21) cbuffer
	Write (3, 21) cbuffer
	Enddo

	Read (2, 21) cbuffer
	Write (3, 36) tso1, tso2, tso3, tso4, tso5, tso6, tso7
 36	Format ('     soil_temperature           = ', 7(f5.1, ', '))
	
	Read (2, 21) cbuffer
	Write (3, 21) cbuffer

	Read (2, 21) cbuffer
	Write (3, 21) cbuffer

	Read (2, 21) cbuffer
	Write (3, 37) tso8
 37	Format ('     deep_soil_temperature      = ' , f5.1, ',')

	Do i = 1, 17
	Read (2, 21) cbuffer
	Write (3, 21) cbuffer
	Enddo

	Stop

	End

